/******************************************************************************
*
* project name:  TICT-Explorer
* file name:     battery.c
* initial date:  31/12/2000
* author:        thomas.nussbaumer@gmx.net
*                Francesco Orabona (bremen79@infinito.it)
*
* description:   battery state evaluation
*
* Credits go to Guillaume de Vivies (gdv@fr.fm) and Loic Minier (Lool@fr.fm)
* for the HW1 source written for TEX
*
*
* $Id: battery.c,v 1.5 2002/02/21 09:13:02 tnussb Exp $
*
******************************************************************************/

//----------------------------------------------------------------------------
// INFORMATIONS TAKEN FROM J89HW.TXT by Johan Eilert <johei804@student.liu.se>
//
// $600000 RW ($00)
//      :7      Keep clear (=0)
//      :6      Keep clear (=0)
//      :5-3    -
//      :2      Battery voltage level is *above* the trig level.
//              (see $600018).
//      :1-0    -
//
//$600018 RW ($0380)
//      :15-10  -
//      :9-7    Battery voltage trigger level (see $600000:2)
//              (HW2: must also enable batt checker at $70001D:3,0)
//                0: ~4.6V, 1: ~4.5V, 2: ~4.3V, 3: ~4.0V,
//                4: ~3.7V, 5: ~3.4V, 6: ~3.2V, (7: ~2.9V  calc resetted!)
//              Remember to wait a while (~250us) before reading from
//              $600000:2 to make sure that the hardware has stabilized.
//              Keep the trig level set to the lowest voltage (%111) when
//              its not in use.  (The keyboard does not work otherwise.)
//              AMS displays "BATT" when the voltage drops below 4.3V and
//              4.0V, respectively.
//      :6-0    Keyboard col mask, bit =1 masks key column (vertical) from
//              being read at $60001B and generate interrupt on key pressed.
//      Note: Some interrupt handlers in AMS writes to this register, so it
//      is a very good idea to disable interrupts first!!!
//
//$70001D RW ($06)
//      :7      Toggles every FS (every time the LCD restarts at line 0)
//      :6-4    -
//      :3      Battery checker bit B (? ???)
//      :2      ? (set)
//      :1      Screen enable (clear this bit to shut down LCD)
//      :0      Battery checker bit A (? enable $600000:2)
//      The battery checker bits must both be set (AB=11) prior to checking
//      the voltage level with $600000:2.  Then, after use, bit B must be
//      cleared (AB=10) while the battery trig hardware settles to the
//      "rest" voltage value (%111).  Finally, both bits should be cleared.
//
//-----------------------------------------------------------------------------


/*===========================================================================*/
/* evaluates battery state (if argument is 0 it assumes HW1 else HW2)        */
/*                                                                           */
/* returns:                                                                  */
/*                                                                           */
/* 7 - full        battery state                                             */
/* 6 - ok          battery state                                             */
/* 5 - medium      battery state                                             */
/* 4 - low         battery state (battery symbol appears)                    */
/* 3 - very low    battery state (battery symbol darkened)                   */
/* 2 - starving    battery state (battery symbol darkened)                   */
/* 1 - almost dead battery state (battery symbol darkened)                   */
/* 0 - dead        battery state (calc resetted)                             */
/*                                                                           */
/* NOTE: The BATT symbol will be turned on by the AMS                        */
/*===========================================================================*/

unsigned short __BatteryState(unsigned short is_hw_2);
asm("__BatteryState:\n"\
"         movem.l  %d2/%a0,-(%sp)\n"\
"         clr.w    %d0\n"\
"         tst.w    12(%sp)       /* jump for HW1 */\n"\
"         beq     __h1_1\n"\
"         movea.l #0x70001D,%a0  /* set a0 to battery checker on register */\n"\
"         move.b  (%a0),%d2\n"\
"         ori.b   #9,%d2\n"\
"         move.b  %d2,(%a0)\n"\
"__h1_1:  movea.l #0x600018,%a0  /* set a0 to trigger level set register */\n"\
"__next:  move.w  %d0,(%a0)      /* set new trigger level */\n"\
"         moveq   #0x6E,%d2      /* loop for a while testing it */\n"\
"__test:  btst.l  #2,(0x600000)  /* check if voltage is above trigger level */\n"\
"         dbra    %d2,__test     /* not only once, but loop for a while ... */\n"\
"         bne     __fine\n"\
"         add.w   #0x80,%d0      /* set another trigger level */\n"\
"         cmp.w   #0x380,%d0\n"\
"         bne     __next\n"\
"__fine:  move.w  #0x380,(%a0)\n"\
"         lsr.w   #7,%d0\n"\
"         tst.w   12(%sp)\n"\
"         beq     __h1_2         /* jump for HW1 */\n"\
"         movea.l #0x70001D,%a0  /* set a0 to battery checker on register */\n"\
"         move.b  (%a0),%d2\n"\
"         andi.b  #0xF7,%d2\n"\
"         move.b  %d2,(%a0)\n"\
"         moveq   #0x52,%d2      /* how long should I loop for testing? */\n"\
"__fin2:  btst.l  #2,(0x600000)  /* check if voltage is above trigger level */\n"\
"         dbne    %d2,__fin2     /* not only once, but loop for a while ... */\n"\
"         move.b  (%a0),%d2\n"\
"         andi.b  #0xF6,%d2\n"\
"         move.b  %d2,(%a0)\n"\
"__h1_2:  movem.l (%sp)+,%d2/%a0\n"\
"         rts");


/*===========================================================================*/
/* battery state fetching routine                                            */
/*===========================================================================*/
unsigned short GetBatteryState(void) {
    unsigned long hwpb, *rombase;
    unsigned short level;
    rombase = (unsigned long *)((*(unsigned long *)0xC8) & 0x600000);
    hwpb = rombase[65];
    OSSetSR(0x700);
    level = __BatteryState(((unsigned short)(hwpb - (unsigned long)rombase  < 0x10000 &&
       *(unsigned short *)hwpb > 22 ? *(unsigned long *)(hwpb + 22) : 1) - 1L));
    OSSetSR(0x0);
    return level;
}


/*===========================================================================*/
/* battery state drawing routine                                             */
/*===========================================================================*/
void DrawBatteryState(short x, short y) {
    unsigned char full_sprite[5]  = { 0x1f,0x1f,0x1f,0x1f,0x1f };
    unsigned char empty_sprite[5] = { 0x1f,0x11,0x11,0x11,0x1f };

    unsigned short bs = GetBatteryState();
    unsigned short i;
    unsigned short bs2;

    const char* msg_level[] = {
        MSGDIRECT_BAT_FULL,
        MSGDIRECT_BAT_OK,
        MSGDIRECT_BAT_MEDIUM,
        MSGDIRECT_BAT_LOW,
        MSGDIRECT_BAT_VERYLOW,
        MSGDIRECT_BAT_ALMOSTDEAD,
        MSGDIRECT_BAT_CHANGE,
        MSGDIRECT_BAT_RESET};

    //bs = random(8);  // for testing ONLY (randomize levels)

    if (bs>5) bs2 = 0;
    else      bs2 = 6 - bs;

    for (i=0;i<6;i++) {
        if (bs2>i)  Sprite8(x+4*i-3,y,5,full_sprite,LCD_MEM,SPRT_OR);
        else        Sprite8(x+4*i-3,y,5,empty_sprite,LCD_MEM,SPRT_OR);
    }

    DrawStr(x+4*7-2,y,msg_level[bs],A_REPLACE);
}


//=============================================================================
// Revision History
//=============================================================================
//
// $Log: battery.c,v $
// Revision 1.5  2002/02/21 09:13:02  tnussb
// completely new battery state detection routine
//
// Revision 1.4  2002/02/07 18:01:17  tnussb
// generic commit
//
// Revision 1.3  2001/02/04 13:11:27  Thomas Nussbaumer
// changes up to version 1.00 RC2 (see history.txt)
//
// Revision 1.2  2001/01/26 21:04:44  Thomas Nussbaumer
// changes for version 0.80 [see history.txt]
//
// Revision 1.1  2001/01/16 03:41:29  Thomas Nussbaumer
// initial version
//
//
//
